@@ -200,6 +200,26 @@ static void xdatastat_notify(GAtResult *result, gpointer user_data)
 	}
 }
 
+static void cpsb_notify(GAtResult *result, gpointer user_data)
+{
+	struct ofono_gprs *gprs = user_data;
+	GAtResultIter iter;
+	gint bearer;
+
+	g_at_result_iter_init(&iter, result);
+
+	if (!g_at_result_iter_next(&iter, "+CPSB:"))
+		return;
+
+	if (!g_at_result_iter_next_number(&iter, NULL))
+		return;
+
+	if (!g_at_result_iter_next_number(&iter, &bearer))
+		return;
+
+	ofono_gprs_bearer_notify(gprs, bearer);
+}
+
 static void gprs_initialized(gboolean ok, GAtResult *result, gpointer user_data)
 {
 	struct ofono_gprs *gprs = user_data;
@@ -207,7 +227,10 @@ static void gprs_initialized(gboolean ok, GAtResult *result, gpointer user_data)
 
 	g_at_chat_register(gd->chat, "+CGEV:", cgev_notify, FALSE, gprs, NULL);
 	g_at_chat_register(gd->chat, "+CGREG:", cgreg_notify,
-					FALSE, gprs, NULL);
+						FALSE, gprs, NULL);
+	g_at_chat_register(gd->chat, "+CPSB:", cpsb_notify, FALSE, gprs, NULL);
+
+	g_at_chat_send(gd->chat, "AT+CPSB=1", none_prefix, NULL, NULL, NULL);
 
 	switch (gd->vendor) {
 	case OFONO_VENDOR_IFX:
@@ -382,12 +405,12 @@ static struct ofono_gprs_driver driver = {
 	.attached_status	= at_gprs_registration_status,
 };
 
-void at_gprs_init()
+void at_gprs_init(void)
 {
 	ofono_gprs_driver_register(&driver);
 }
 
-void at_gprs_exit()
+void at_gprs_exit(void)
 {
 	ofono_gprs_driver_unregister(&driver);
 }
