@@ -662,13 +662,13 @@ static void ifx_xciev_notify(GAtResult *result, gpointer user_data)
 		return;
 
 	if (ind == 0)
-		strength = 0;
+		strength = -1;
 	else if (ind == 7)
 		strength = 100;
 	else
 		strength = (ind * 15);
 
-	ofono_netreg_strength_notify(netreg, ind);
+	ofono_netreg_strength_notify(netreg, strength);
 }
 
 static void ciev_notify(GAtResult *result, gpointer user_data)
@@ -698,8 +698,8 @@ static void ciev_notify(GAtResult *result, gpointer user_data)
 
 static void ctzv_notify(GAtResult *result, gpointer user_data)
 {
-	//struct ofono_netreg *netreg = user_data;
-	//struct netreg_data *nd = ofono_netreg_get_data(netreg);
+	struct ofono_netreg *netreg = user_data;
+	struct netreg_data *nd = ofono_netreg_get_data(netreg);
 	const char *tz;
 	GAtResultIter iter;
 
@@ -712,6 +712,10 @@ static void ctzv_notify(GAtResult *result, gpointer user_data)
 		return;
 
 	DBG("tz %s", tz);
+
+	nd->time.utcoff = atoi(tz) * 15 * 60;
+
+	ofono_netreg_time_notify(netreg, &nd->time);
 }
 
 static void ifx_ctzv_notify(GAtResult *result, gpointer user_data)
@@ -1342,12 +1346,12 @@ static struct ofono_netreg_driver driver = {
 	.strength			= at_signal_strength,
 };
 
-void at_netreg_init()
+void at_netreg_init(void)
 {
 	ofono_netreg_driver_register(&driver);
 }
 
-void at_netreg_exit()
+void at_netreg_exit(void)
 {
 	ofono_netreg_driver_unregister(&driver);
 }
