@@ -137,6 +137,7 @@ static gboolean check_sim_status(const GIsiMessage *msg, uint8_t msgid,
 		DBG("Request failed: %s", sim_isi_cause_name(cause));
 		return FALSE;
 	}
+
 	return TRUE;
 }
 
@@ -221,6 +222,7 @@ static void sca_query_resp_cb(const GIsiMessage *msg, void *data)
 		goto error;
 
 	bcd_len = info->sca[0];
+
 	if (bcd_len == 0 || bcd_len > 12)
 		goto error;
 
@@ -249,8 +251,8 @@ static void isi_sca_query(struct ofono_sms *sms,
 	if (cbd == NULL || sd == NULL)
 		goto error;
 
-	if (g_isi_client_send(sd->sim, msg, sizeof(msg), SIM_TIMEOUT,
-				sca_query_resp_cb, cbd, g_free) != NULL)
+	if (g_isi_client_send(sd->sim, msg, sizeof(msg),
+				sca_query_resp_cb, cbd, g_free))
 		return;
 
 error:
@@ -300,8 +302,7 @@ static void isi_sca_set(struct ofono_sms *sms,
 	bcd[0] = 1 + (strlen(sca->number) + 1) / 2;
 	bcd[1] = sca->type & 0xFF;
 
-	if (g_isi_client_vsend(sd->sim, iov, 2, SIM_TIMEOUT, sca_set_resp_cb,
-				cbd, g_free) != NULL)
+	if (g_isi_client_vsend(sd->sim, iov, 2, sca_set_resp_cb, cbd, g_free))
 		return;
 
 error:
@@ -327,7 +328,7 @@ static void submit_resp_cb(const GIsiMessage *msg, void *data)
 		if (g_isi_sb_iter_get_id(&iter) != SMS_GSM_REPORT)
 			continue;
 
-		if (!g_isi_sb_iter_get_struct(&iter, (void **)&report, len, 2))
+		if (!g_isi_sb_iter_get_struct(&iter, (void **) &report, len, 2))
 			goto error;
 
 		if (report->cause != SMS_OK)
@@ -403,8 +404,9 @@ static void isi_submit(struct ofono_sms *sms, unsigned char *pdu,
 	 * Modem seems to time out SMS_MESSAGE_SEND_REQ in 5 seconds.
 	 * Wait normal timeout plus the modem timeout.
 	 */
-	if (g_isi_client_vsend(sd->client, iov, 4, SMS_TIMEOUT + 5,
-				submit_resp_cb, cbd, g_free) != NULL)
+	if (g_isi_client_vsend_with_timeout(sd->client, iov, 4,
+				SMS_TIMEOUT + 5,
+				submit_resp_cb, cbd, g_free))
 		return;
 
 error:
@@ -436,7 +438,7 @@ static void send_status_ind_cb(const GIsiMessage *msg, void *data)
 	if (g_isi_msg_id(msg) != SMS_MESSAGE_SEND_STATUS_IND)
 		return;
 
-	if (!g_isi_msg_data_get_struct(msg, 0, (const void **)&info, len))
+	if (!g_isi_msg_data_get_struct(msg, 0, (const void **) &info, len))
 		return;
 
 	DBG("status=0x%"PRIx8", ref=0x%"PRIx8", route=0x%"PRIx8
@@ -481,9 +483,9 @@ static gboolean send_deliver_report(GIsiClient *client, gboolean success)
 		0, 0, 0,	/* Filler */
 		0,		/* Sub blocks */
 	};
+	size_t len = sizeof(msg);
 
-	return g_isi_client_send(client, msg, sizeof(msg), SMS_TIMEOUT,
-					report_resp_cb, NULL, NULL) != NULL;
+	return g_isi_client_send(client, msg, len, report_resp_cb, NULL, NULL);
 }
 
 static gboolean parse_sms_address(GIsiSubBlockIter *iter, struct sms_addr *add)
@@ -538,15 +540,18 @@ static gboolean parse_gsm_tpdu(GIsiSubBlockIter *parent, struct sms_addr *add,
 
 			if (add->type != SMS_GSM_0411_ADDRESS)
 				return FALSE;
+
 			break;
 
 		case SMS_COMMON_DATA:
 
 			if (!parse_sms_tpdu(&iter, com))
 				return FALSE;
+
 			break;
 		}
 	}
+
 	return TRUE;
 }
 
@@ -606,12 +611,12 @@ static void routing_resp_cb(const GIsiMessage *msg, void *data)
 	ofono_sms_register(sms);
 }
 
-static int isi_sms_probe(struct ofono_sms *sms, unsigned int vendor,
-				void *user)
+static void sim_reachable_cb(const GIsiMessage *msg, void *data)
 {
-	GIsiModem *modem = user;
-	struct sms_data *sd = g_try_new0(struct sms_data, 1);
-	const uint8_t msg[] = {
+	struct ofono_sms *sms = data;
+	struct sms_data *sd = ofono_sms_get_data(sms);
+
+	const uint8_t req[] = {
 		SMS_PP_ROUTING_REQ,
 		SMS_ROUTING_SET,
 		0x01,  /* Sub-block count */
@@ -622,6 +627,39 @@ static int isi_sms_probe(struct ofono_sms *sms, unsigned int vendor,
 		0x00, 0x00, 0x00,  /* Filler */
 		0x00  /* Sub-sub-block count */
 	};
+	size_t len = sizeof(req);
+
+	if (g_isi_msg_error(msg) < 0) {
+		DBG("unable to find SIM resource");
+		g_isi_client_destroy(sd->sim);
+		sd->sim = NULL;
+	}
+
+	g_isi_client_ind_subscribe(sd->client, SMS_MESSAGE_SEND_STATUS_IND,
+					send_status_ind_cb, sms);
+	g_isi_client_send(sd->client, req, len, routing_resp_cb, sms, NULL);
+}
+
+static void sms_reachable_cb(const GIsiMessage *msg, void *data)
+{
+	struct ofono_sms *sms = data;
+	struct sms_data *sd = ofono_sms_get_data(sms);
+
+	if (g_isi_msg_error(msg) < 0) {
+		DBG("unable to find SMS resource");
+		return;
+	}
+
+	ISI_VERSION_DBG(msg);
+
+	g_isi_client_verify(sd->sim, sim_reachable_cb, sms, NULL);
+}
+
+static int isi_sms_probe(struct ofono_sms *sms, unsigned int vendor,
+				void *user)
+{
+	GIsiModem *modem = user;
+	struct sms_data *sd = g_try_new0(struct sms_data, 1);
 
 	if (sd == NULL)
 		return -ENOMEM;
@@ -631,22 +669,25 @@ static int isi_sms_probe(struct ofono_sms *sms, unsigned int vendor,
 
 	sd->client = g_isi_client_create(modem, PN_SMS);
 	if (sd->client == NULL)
-		return -ENOMEM;
+		goto nomem;
 
 	sd->sim = g_isi_client_create(modem, PN_SIM);
-	if (sd->sim == NULL) {
-		g_isi_client_destroy(sd->client);
-		return -ENOMEM;
-	}
+	if (sd->sim == NULL)
+		goto nomem;
+
+	g_isi_client_set_timeout(sd->client, SMS_TIMEOUT);
+	g_isi_client_set_timeout(sd->sim, SIM_TIMEOUT);
 
 	ofono_sms_set_data(sms, sd);
 
-	g_isi_client_ind_subscribe(sd->client, SMS_MESSAGE_SEND_STATUS_IND,
-					send_status_ind_cb, sms);
-	g_isi_client_send(sd->client, msg, sizeof(msg), SMS_TIMEOUT,
-				routing_resp_cb, sms, NULL);
+	g_isi_client_verify(sd->client, sms_reachable_cb, sms, NULL);
 
 	return 0;
+
+nomem:
+	g_isi_client_destroy(sd->client);
+	g_free(sd);
+	return -ENOMEM;
 }
 
 static void isi_sms_remove(struct ofono_sms *sms)
@@ -674,8 +715,7 @@ static void isi_sms_remove(struct ofono_sms *sms)
 	 * Send a promiscuous routing release, so as not to
 	 * hog resources unnecessarily after being removed
 	 */
-	g_isi_client_send(sd->client, msg, sizeof(msg), SMS_TIMEOUT,
-				NULL, NULL, NULL);
+	g_isi_client_send(sd->client, msg, sizeof(msg), NULL, NULL, NULL);
 	g_isi_client_destroy(sd->client);
 	g_isi_client_destroy(sd->sim);
 	g_free(sd);
@@ -692,12 +732,12 @@ static struct ofono_sms_driver driver = {
 	.bearer_set		= isi_bearer_set,
 };
 
-void isi_sms_init()
+void isi_sms_init(void)
 {
 	ofono_sms_driver_register(&driver);
 }
 
-void isi_sms_exit()
+void isi_sms_exit(void)
 {
 	ofono_sms_driver_unregister(&driver);
 }
