@@ -289,7 +289,8 @@ static void sysinfo_cb(gboolean ok, GAtResult *result, gpointer user_data)
 		data->sim_poll_timeout = g_timeout_add_seconds(2,
 								query_sim_state,
 								modem);
-	} else if (sim_state ==	HUAWEI_SIM_STATE_INVALID_OR_LOCKED) {
+	} else if (sim_state ==	HUAWEI_SIM_STATE_INVALID_OR_LOCKED &&
+						!data->sim_poll_timeout) {
 		data->sim_poll_timeout = g_timeout_add_seconds(2,
 								query_sim_locked,
 								modem);
@@ -618,11 +619,8 @@ static void huawei_pre_sim(struct ofono_modem *modem)
 	DBG("%p", modem);
 
 	ofono_devinfo_create(modem, 0, "atmodem", data->pcui);
-	data->sim = ofono_sim_create(modem, OFONO_VENDOR_QUALCOMM_MSM,
+	data->sim = ofono_sim_create(modem, OFONO_VENDOR_HUAWEI,
 					"atmodem", data->pcui);
-
-	data->sim_poll_count = 0;
-	query_sim_state(modem);
 }
 
 static void huawei_post_sim(struct ofono_modem *modem)
