@@ -262,6 +262,30 @@ gboolean valid_phone_number_format(const char *number)
 	return TRUE;
 }
 
+gboolean valid_cdma_phone_number_format(const char *number)
+{
+	int len = strlen(number);
+	int i;
+
+	if (!len)
+		return FALSE;
+
+	if (len > OFONO_CDMA_MAX_PHONE_NUMBER_LENGTH)
+		return FALSE;
+
+	for (i = 0; i < len; i++) {
+		if (number[i] >= '0' && number[i] <= '9')
+			continue;
+
+		if (number[i] == '*' || number[i] == '#')
+			continue;
+
+		return FALSE;
+	}
+
+	return TRUE;
+}
+
 const char *telephony_error_to_str(const struct ofono_error *error)
 {
 	struct error_entry *e;
@@ -336,27 +360,39 @@ int mmi_service_code_to_bearer_class(int code)
 	case 19:
 		cls = BEARER_CLASS_VOICE | BEARER_CLASS_FAX;
 		break;
-
-	/* 22.030: 7-11 */
-	/* 22.004 only defines BS 7 (Data Sync) & BS 8 (Data Async) */
+	/*
+	 * 22.030: 7-11
+	 * 22.004 only defines BS 7 (Data Sync) & BS 8 (Data Async)
+	 * and PAD and Packet bearer services are deprecated.  Still,
+	 * AT modems rely on these to differentiate between sending
+	 * a 'All Sync' or 'All Data Sync' message types.  In theory
+	 * both message types cover the same bearer services, but we
+	 * must still send these for conformance reasons.
+	 */
 	case 20:
-		cls = BEARER_CLASS_DATA_ASYNC | BEARER_CLASS_DATA_SYNC;
+		cls = BEARER_CLASS_DATA_ASYNC | BEARER_CLASS_DATA_SYNC |
+			BEARER_CLASS_PAD | BEARER_CLASS_PACKET;
 		break;
 	/* According to 22.030: All Async (7) */
 	case 21:
+		cls = BEARER_CLASS_DATA_ASYNC | BEARER_CLASS_PAD;
+		break;
 	/* According to 22.030: All Data Async (7)*/
 	case 25:
 		cls = BEARER_CLASS_DATA_ASYNC;
 		break;
 	/* According to 22.030: All Sync (8) */
 	case 22:
+		cls = BEARER_CLASS_DATA_SYNC | BEARER_CLASS_PACKET;
+		break;
 	/* According to 22.030: All Data Sync (8) */
 	case 24:
 		cls = BEARER_CLASS_DATA_SYNC;
 		break;
 	/* According to 22.030: Telephony & All Sync services (1, 8) */
 	case 26:
-		cls = BEARER_CLASS_VOICE | BEARER_CLASS_DATA_SYNC;
+		cls = BEARER_CLASS_VOICE | BEARER_CLASS_DATA_SYNC |
+			BEARER_CLASS_PACKET;
 		break;
 	default:
 		break;
@@ -384,7 +420,7 @@ const char *phone_number_to_string(const struct ofono_phone_number *ph)
 
 void string_to_phone_number(const char *str, struct ofono_phone_number *ph)
 {
-	if (strlen(str) && str[0] == '+') {
+	if (str[0] == '+') {
 		strcpy(ph->number, str+1);
 		ph->type = 145;	/* International */
 	} else {
@@ -393,6 +429,23 @@ void string_to_phone_number(const char *str, struct ofono_phone_number *ph)
 	}
 }
 
+const char *cdma_phone_number_to_string(
+				const struct ofono_cdma_phone_number *ph)
+{
+	static char buffer[OFONO_CDMA_MAX_PHONE_NUMBER_LENGTH + 1];
+
+	strncpy(buffer, ph->number, OFONO_CDMA_MAX_PHONE_NUMBER_LENGTH);
+	buffer[OFONO_CDMA_MAX_PHONE_NUMBER_LENGTH] = '\0';
+
+	return buffer;
+}
+
+void string_to_cdma_phone_number(const char *str,
+					struct ofono_cdma_phone_number *ph)
+{
+	strcpy(ph->number, str);
+}
+
 gboolean valid_ussd_string(const char *str, gboolean call_in_progress)
 {
 	int len = strlen(str);
@@ -504,7 +557,7 @@ gboolean parse_ss_control_string(char *str, int *ss_type,
 	/* Must have at least one other '#' */
 	c = strrchr(str+cur, '#');
 
-	if (!c)
+	if (c == NULL)
 		goto out;
 
 	*dn = c+1;
@@ -661,6 +714,29 @@ const char *registration_tech_to_string(int tech)
 	}
 }
 
+const char *packet_bearer_to_string(int bearer)
+{
+	switch (bearer) {
+	case PACKET_BEARER_NONE:
+		return "none";
+	case PACKET_BEARER_GPRS:
+		return "gprs";
+	case PACKET_BEARER_EGPRS:
+		return "edge";
+	case PACKET_BEARER_UMTS:
+		return "umts";
+	case PACKET_BEARER_HSUPA:
+		return "hsupa";
+	case PACKET_BEARER_HSDPA:
+		return "hsdpa";
+	case PACKET_BEARER_HSUPA_HSDPA:
+		return "hspa";
+	case PACKET_BEARER_EPS:
+		return "lte";
+	}
+	return "";
+}
+
 gboolean is_valid_apn(const char *apn)
 {
 	int i;
