@@ -67,6 +67,7 @@ struct ofono_gprs {
 	ofono_bool_t suspended;
 	int status;
 	int flags;
+	int bearer;
 	guint suspend_timeout;
 	struct idmap *pid_map;
 	unsigned int last_context_id;
@@ -175,7 +176,7 @@ static gboolean gprs_context_string_to_type(const char *str,
 		return TRUE;
 	} else if (g_str_equal(str, "ims")) {
 		*out = OFONO_GPRS_CONTEXT_TYPE_IMS;
-		return FALSE;
+		return TRUE;
 	}
 
 	return FALSE;
@@ -336,7 +337,7 @@ static void pri_context_signal_settings(struct pri_context *ctx)
 					OFONO_CONNECTION_CONTEXT_INTERFACE,
 					"PropertyChanged");
 
-	if (!signal)
+	if (signal == NULL)
 		return;
 
 	dbus_message_iter_init_append(signal, &iter);
@@ -400,7 +401,7 @@ static void pri_ifupdown(const char *interface, ofono_bool_t active)
 	struct ifreq ifr;
 	int sk;
 
-	if (!interface)
+	if (interface == NULL)
 		return;
 
 	sk = socket(PF_INET, SOCK_DGRAM, 0);
@@ -436,7 +437,7 @@ static void pri_setaddr(const char *interface, const char *address)
 	struct sockaddr_in addr;
 	int sk;
 
-	if (!interface)
+	if (interface == NULL)
 		return;
 
 	sk = socket(PF_INET, SOCK_DGRAM, 0);
@@ -459,7 +460,7 @@ static void pri_setaddr(const char *interface, const char *address)
 		goto done;
 	}
 
-	if (!address)
+	if (address == NULL)
 		goto done;
 
 	memset(&addr, 0, sizeof(addr));
@@ -480,7 +481,7 @@ static void pri_setproxy(const char *interface, const char *proxy)
 	struct sockaddr_in addr;
 	int sk;
 
-	if (!interface)
+	if (interface == NULL)
 		return;
 
 	sk = socket(PF_INET, SOCK_DGRAM, 0);
@@ -550,7 +551,7 @@ static void pri_update_context_settings(struct pri_context *ctx,
 		context_settings_free(ctx->settings);
 
 	ctx->settings = g_try_new0(struct context_settings, 1);
-	if (!ctx->settings)
+	if (ctx->settings == NULL)
 		return;
 
 	ctx->settings->type = ctx->type;
@@ -633,7 +634,7 @@ static DBusMessage *pri_get_properties(DBusConnection *conn,
 	DBusMessageIter dict;
 
 	reply = dbus_message_new_method_return(msg);
-	if (!reply)
+	if (reply == NULL)
 		return NULL;
 
 	dbus_message_iter_init_append(reply, &iter);
@@ -1147,13 +1148,15 @@ static struct pri_context *pri_context_create(struct ofono_gprs *gprs,
 {
 	struct pri_context *context = g_try_new0(struct pri_context, 1);
 
-	if (!context)
+	if (context == NULL)
 		return NULL;
 
-	if (!name) {
+	if (name == NULL) {
 		name = gprs_context_default_name(type);
-		if (!name)
+		if (name == NULL) {
+			g_free(context);
 			return NULL;
+		}
 	}
 
 	context->gprs = gprs;
@@ -1317,6 +1320,8 @@ static void gprs_attached_update(struct ofono_gprs *gprs)
 					OFONO_CONNECTION_CONTEXT_INTERFACE,
 					"Active", DBUS_TYPE_BOOLEAN, &value);
 		}
+
+		gprs->bearer = -1;
 	}
 
 	path = __ofono_atom_get_path(gprs->atom);
@@ -1419,7 +1424,7 @@ static DBusMessage *gprs_get_properties(DBusConnection *conn,
 	dbus_bool_t value;
 
 	reply = dbus_message_new_method_return(msg);
-	if (!reply)
+	if (reply == NULL)
 		return NULL;
 
 	dbus_message_iter_init_append(reply, &iter);
@@ -1431,6 +1436,13 @@ static DBusMessage *gprs_get_properties(DBusConnection *conn,
 	value = gprs->attached;
 	ofono_dbus_dict_append(&dict, "Attached", DBUS_TYPE_BOOLEAN, &value);
 
+	if (gprs->bearer != -1) {
+		const char *bearer = packet_bearer_to_string(gprs->bearer);
+
+		ofono_dbus_dict_append(&dict, "Bearer",
+					DBUS_TYPE_STRING, &bearer);
+	}
+
 	value = gprs->roaming_allowed;
 	ofono_dbus_dict_append(&dict, "RoamingAllowed",
 				DBUS_TYPE_BOOLEAN, &value);
@@ -1497,7 +1509,7 @@ static DBusMessage *gprs_set_property(DBusConnection *conn,
 
 		gprs_netreg_update(gprs);
 	} else if (!strcmp(property, "Powered")) {
-		if (!gprs->driver->set_attached)
+		if (gprs->driver->set_attached == NULL)
 			return __ofono_error_not_implemented(msg);
 
 		if (dbus_message_iter_get_arg_type(&var) != DBUS_TYPE_BOOLEAN)
@@ -1563,7 +1575,7 @@ static struct pri_context *add_context(struct ofono_gprs *gprs,
 		return NULL;
 
 	context = pri_context_create(gprs, name, type);
-	if (!context) {
+	if (context == NULL) {
 		ofono_error("Unable to allocate context struct");
 		return NULL;
 	}
@@ -1667,6 +1679,8 @@ static void gprs_deactivate_for_remove(const struct ofono_error *error,
 
 	gprs_cid_release(gprs, ctx->context.cid);
 	ctx->context.cid = 0;
+	ctx->context_driver->inuse = FALSE;
+	ctx->context_driver = NULL;
 
 	if (gprs->settings) {
 		g_key_file_remove_group(gprs->settings, ctx->key, NULL);
@@ -1709,7 +1723,7 @@ static DBusMessage *gprs_remove_context(DBusConnection *conn,
 		return __ofono_error_invalid_format(msg);
 
 	ctx = gprs_context_by_path(gprs, path);
-	if (!ctx)
+	if (ctx == NULL)
 		return __ofono_error_not_found(msg);
 
 	if (ctx->active) {
@@ -1730,7 +1744,7 @@ static DBusMessage *gprs_remove_context(DBusConnection *conn,
 		storage_sync(gprs->imsi, SETTINGS_STORE, gprs->settings);
 	}
 
-	DBG("Unregistering context: %s\n", ctx->path);
+	DBG("Unregistering context: %s", ctx->path);
 	context_dbus_unregister(ctx);
 	gprs->contexts = g_slist_remove(gprs->contexts, ctx);
 
@@ -1977,6 +1991,23 @@ void ofono_gprs_add_context(struct ofono_gprs *gprs,
 	__ofono_atom_register(gc->atom, gprs_context_unregister);
 }
 
+void ofono_gprs_bearer_notify(struct ofono_gprs *gprs, int bearer)
+{
+	DBusConnection *conn = ofono_dbus_get_connection();
+	const char *path;
+	const char *value;
+
+	if (gprs->bearer == bearer)
+		return;
+
+	gprs->bearer = bearer;
+	path = __ofono_atom_get_path(gprs->atom);
+	value = packet_bearer_to_string(bearer);
+	ofono_dbus_signal_property_changed(conn, path,
+					OFONO_CONNECTION_CONTEXT_INTERFACE,
+					"Bearer", DBUS_TYPE_STRING, &value);
+}
+
 void ofono_gprs_context_deactivated(struct ofono_gprs_context *gc,
 					unsigned int cid)
 {
@@ -1991,12 +2022,12 @@ void ofono_gprs_context_deactivated(struct ofono_gprs_context *gc,
 	for (l = gc->gprs->contexts; l; l = l->next) {
 		ctx = l->data;
 
-		if (ctx->active == FALSE)
-			continue;
-
 		if (ctx->context.cid != cid)
 			continue;
 
+		if (ctx->active == FALSE)
+			break;
+
 		gprs_cid_release(ctx->gprs, ctx->context.cid);
 		ctx->context.cid = 0;
 		ctx->active = FALSE;
